//
//  EMBSnapDocument.h
//  Ember
//
//  Created by Damien DeVille on 24/05/2012.
//  Copyright (c) 2012 Realmac Software. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "EMBEmberData/EMBSnapDocument+Content.h"
#import "EMBEmberData/EMBEmberFileManager.h"

@interface EMBSnapDocument : NSObject <EMBSnapInfo, EMBSnapMetadata, EMBSnapAnnotations>

/*!
	\brief
	Initializer for a new document, not yet on disk.
 */
- (id)initForSavingToURL:(NSURL *)fileURL;

/*!
	\brief
	Initializer for an existing document on disk, located at `fileURL`.
 */
- (id)initWithContentsOfURL:(NSURL *)fileURL error:(NSError **)errorRef;

/*!
	\brief
	The location of the document on disk. For new documents, it will be nil until the document has been saved.
 */
@property (readonly, copy, nonatomic) NSURL *fileURL;

/*!
	\brief
	The document file type.
 */
- (NSString *)fileType;

/*!
	The file manager associated with the document. It should be use for any file system related access with the document.
 */
@property (readonly, strong, nonatomic) id <EMBEmberFileManager> fileManager;

/*!
	\brief
	Save the document to the given location on disk.
 */
- (BOOL)saveToURL:(NSURL *)fileURL error:(NSError **)errorRef;

/*!
	\brief
	Determine whether the document has unsaved changes.
 */
@property (readonly, assign, getter = isDirty, nonatomic) BOOL dirty;

/*!
	\brief
	Update the data from the representation on disk.
 */
- (BOOL)refreshFromDisk:(NSError **)errorRef;

/*!
	\brief
	The snap document undo manager.
 */
@property (readonly, strong, nonatomic) NSUndoManager *undoManager;

/*!
	\brief
	The absolute location of the snap image.
 */
@property (readonly, copy, nonatomic) NSURL *snapMediaURL;
- (BOOL)setSnapMediaURL:(NSURL *)snapImageURL error:(NSError **)errorRef;

/*!
	\brief
	Creates an image source for the snap document image.
 */
- (CGImageSourceRef)createSnapImageSource:(NSDictionary *)options error:(NSError **)errorRef CF_RETURNS_RETAINED;

/*!
	\brief
	The absolute location of the snap composited image.
 */
@property (readonly, copy, nonatomic) NSURL *snapCompositedImageURL;
- (BOOL)setSnapCompositedImageURL:(NSURL *)snapCompositedImageURL error:(NSError **)errorRef;

/*!
	\brief
	The absolute location of the snap web archive.
 */
@property (readonly, copy, nonatomic) NSURL *snapWebArchiveURL;
- (BOOL)setSnapWebArchiveURL:(NSURL *)snapWebArchiveURL error:(NSError **)errorRef;

/*!
	\brief
	Add a window layer image at a given URL. Returns the local document path for this window layer.
 */
- (NSString *)addWindowLayerImageURL:(NSURL *)windowLayerImageURL error:(NSError **)errorRef;

/*!
	\brief
	The size in pixel of the snap image, taking annotations into account.
 */
@property (readonly, assign, nonatomic) CGSize snapPixelSize;

/*!
	\brief
	Add a new annotation.
 */
- (void)addAnnotation:(EMBAnnotation *)annotation;

/*!
	\brief
	Remove an existing annotation.
 */
- (void)removeAnnotation:(EMBAnnotation *)annotation;

/*!
	\brief
	Update an existing annotation.
 */
- (void)updateAnnotation:(EMBAnnotation *)annotation;

/*!
	\brief
	A snap is seen as annotated when at least one of the following is true:
		- It contains at least one annotation.
		- It is rotated.
		- It is cropped.
 */
- (BOOL)isAnnotated;

@end

@class EMBSnap;

@interface EMBSnapDocument (EMBSnap)

/*!
	\brief
	Update the snap document metadata based on the values stored in the snap managed object.
 */
- (void)updateMetadataWithSnap:(EMBSnap *)snap;

@end

@interface EMBSnapDocument (EMBAnnotationTreeProvider)

/*!
	\brief
	Create a new annotation tree provider based on the snap document content.
 */
- (EMBAnnotationTreeProvider *)makeAnnotationTreeProvider:(NSError **)errorRef;

@end

@interface EMBSnapDocument (LittleSnapperSupport) <EMBSnapDocumentLittleSnapperSupport>

@end

@interface EMBSnapDocument (QuickLook)

/*!
	\brief
	Returns YES if the document at the given location has quicklook setup.
 */
- (BOOL)isQuicklookSetup;

@end
